/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation;

import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationStrategy;

import com.google.common.collect.Multimap;

/**
 * Base class for generating AF3 models programmatically.
 * <p>
 * <b>NOTE:</b> This class will only be present during the transition phase to the new model
 * generation/transformation "framework". DO NOT USE ELSEWHERE!!!
 * 
 * @author diewald
 */
public abstract class DeprecatedModelGeneratorBase<T extends EObject> implements
		ITransformationStrategy<T> {

	/**
	 * Associative map which relates elements of a generated model to those of the original input
	 * model.
	 */
	protected Map<EObject, EObject> genToOrigAssocMap;
	/**
	 * Associative map which relates elements of an original input model to those of a generated
	 * model.
	 */
	protected Multimap<EObject, EObject> origToGenAssocMap;

	/** Constructor. */
	protected DeprecatedModelGeneratorBase(Map<EObject, EObject> genToOrigAssocMap,
			Multimap<EObject, EObject> origToGenAssocMap) {
		this.genToOrigAssocMap = genToOrigAssocMap;
		this.origToGenAssocMap = origToGenAssocMap;
	}

	// /** {@inheritDoc} */
	// @Override
	// public Map<EObject, EObject> getGenToOrigAssociations() {
	// return genToOrigAssocMap;
	// }
	//
	// /** {@inheritDoc} */
	// @Override
	// public Multimap<EObject, EObject> getOrigToGenAssociations() {
	// return origToGenAssocMap;
	// }
}
