/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.arch.af3;

import static org.fortiss.af3.exploration.alg.util.AF3Utils.getAnnotationChecked;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.exploration.alg.exception.InvalidModelException;
import org.fortiss.af3.platform.model.MemoryUnit;
import org.fortiss.af3.platform.model.PlatformArchitecture;
import org.fortiss.af3.platform.model.annotation.MemorySize;

/**
 * {@link IMemoryUnitAdapter} for memory resources of an AF3 {@link PlatformArchitecture}.
 * 
 * @author diewald
 */
public class AF3MemoryUnitAdapter extends AF3ResourceAdapter<MemoryUnit>
		implements IMemoryUnitAdapter<MemoryUnit> {

	/** References the {@link MemorySize} annotation. */
	private MemorySize memorySizeSpec;

	/**
	 * Constructor.
	 * 
	 * @param modelElement
	 *            {@link MemoryUnit} from the target platform.
	 * 
	 * @throws InvalidModelException
	 *             if a required annotation cannot be found.
	 */
	public AF3MemoryUnitAdapter(MemoryUnit modelElement) throws InvalidModelException {
		super(modelElement);
		memorySizeSpec = getAnnotationChecked(modelElement, MemorySize.class);
	}

	/** {@inheritDoc} */
	@Override
	public long getSizeInBytes() {
		return memorySizeSpec.getSize();
	}
}
